from gi.repository import Gtk
import os.path
from .conffile import Config
from .alsamorse import KochLesson

class PropertyDialog:
    def __init__(self):
        self.__gladefile = os.path.join(os.path.dirname(__file__), "propdialog.ui")
        self.__xml = Gtk.Builder()
        self.__xml.add_from_file(self.__gladefile)
        self.__widget = self.__xml.get_object("propdialog")
        self.__lessontime = self.__xml.get_object("lessontime")
        self.__countdown  = self.__xml.get_object("countdown")
        self.__prefer_last = self.__xml.get_object("prefer_last")
        self.__ign_box = self.__xml.get_object("ignorechars")
        
        cfg = Config()
        self.__lessontime.set_value(cfg.opt_lessontime)
        self.__countdown.set_value(cfg.opt_countdown)
        self.__prefer_last.set_active(cfg.opt_preferlast)
        self.__ign_box.set_text(cfg.opt_ignorechars)
        
            

    def run(self):
        if self.__widget.run() == Gtk.ResponseType.CANCEL:
            self.__widget.destroy()
            return None

        cfg = Config()
        cfg.opt_lessontime  = self.__lessontime.get_value()
        cfg.opt_countdown   = self.__countdown.get_value()
        cfg.opt_preferlast  = self.__prefer_last.get_active()
        cfg.opt_ignorechars = filter(lambda x: x in KochLesson.chars,
                                     self.__ign_box.get_text())
        cfg.save()

        self.__widget.destroy()

                




import alsaaudio
import struct
from math import sin, pi, ceil, floor
from threading import Thread, Event, Lock
import time
import logging


class Morse(Thread):
    """ This class implements the morse-engine. Instancing this class will 
        create a thread that does all the sound-io. This thread will initial 
        sleep untill Morse.pause(False) is called. You can reset the engine 
        into pause by calling Morse.pause(True). 

        Note; Since this class starts a new thread, you have to call 
        Morse.stop() to stop and join this thread.

        Morse can use anything that is iterratable as a char-source. """

    __morse_table = {'a':'.-',     'b':'-...',   'c':'-.-.',   'd':'-..',
                     'e':'.',      'f':'..-.',   'g':'--.',    'h':'....',
                     'i':'..',     'j':'.---',   'k':'-.-',    'l':'.-..',
                     'm':'--',     'n':'-.',     'o':'---',    'p':'.--.',
                     'q':'--.-',   'r':'.-.',    's':'...',    't':'-',
                     'u':'..-',    'v':'...-',   'w':'.--',    'x':'-..-',
                     'y':'-.--',   'z':'--..',   '0':'-----',  '1':'.----',
                     '2':'..---',  '3':'...--',  '4':'....-',  '5':'.....',
                     '6':'-....',  '7':'--...',  '8':'---..',  '9':'----.',
                     '.':'.-.-.-', ',':'--..--', '?':'..--..', "'":'.----.',
                     '!':'-.-.--', '/':'-..-.',  '(':'-.--.',  ')':'-.--.-',
                     '&':'.-...',  ':':'---...', ';':'-.-.-.', '=':'-...-',
                     '+':'.-.-.',  '-':'-....-', '_':'..--.-', '"':'.-..-.',
                     '$':'...-..-','@':'.--.-.', 
                     'VE':'...-.', 'SK':'...-.-'}


    def __init__(self, frequency=750.0, speed=25.0, eff_speed=15.0, volume=0):
        """ The constructor takes 4 arguments. freqeucny specifies the 
            tone-frequency, speed the coding-speed, eff_speed the effective 
            conding-speed and volume the volumen in dB.

            @param frequency: Tone-frequence used in Hz. By default 750.0 Hz.
            @param speed: Coding-speed in WPM (words per minute). This 
                parameter specifies how fast a charater is send. The 
                pause-length between chars and words are defined by the 
                eff_speed parameter. 
            @param eff_speed: Effecive code speed in WPM. This speed defines 
                the length of the pauses between chars and words. This makes 
                it possible to train at full speed and have enougth time to 
                write chars down. 
            @param volume: Specifies the volume in dB. By default 0, which 
                means maximum."""
            
        self.__logger = logging.getLogger("kochmorse")
        self.__logger.info("Init morse with speed %s(%s), volume %s and freq %s"%(speed, eff_speed, volume, frequency))
        assert volume <= 0
        
        # call constructor
        Thread.__init__(self)

        # store attr:
        self.__frequency = float(frequency)
        self.__speed     = float(speed)
        self.__eff_speed = float(eff_speed)
        self.__volume    = 10.0**(float(volume)/20.0) * 32767.0

        self.__dit_sample = str()
        self.__da_sample  = str()
        self.__char_pause = str()
        self.__word_pause = str()

        # open alsa-pcm device for output:        
        self.__pcm = alsaaudio.PCM(alsaaudio.PCM_PLAYBACK, alsaaudio.PCM_NONBLOCK)
        #self.__pcm = WaveWriter("test.wav")

        self.__sample_rate = 48000.0

        self.__pcm.setchannels(1)
        self.__pcm.setrate(int(self.__sample_rate))
        self.__pcm.setformat(alsaaudio.PCM_FORMAT_S16_LE)
     
        self.__sample_lock = Lock()
        self.__generate_samples()
 
        # thread stuff
        self.__running = True
        self.__pause = Event()

        self.__char_source = None
        self.__char_source_lock = Lock()
 
        self.__char_notifier = None
 
        # start thread
        self.start()
 

    def __generate_samples(self): 
        self.__sample_lock.acquire()
        
        # pre-calculate some stuff:
        omega = 2.0*pi*self.__frequency/self.__sample_rate  # jupp it is Omega
        period_len = 2.0*self.__sample_rate/self.__frequency    # period in frames

        # calculate sample-lengths: 
        self.__dit_len        = (60.0 * self.__sample_rate) / (50.0 * float(self.__speed))
        self.__dit_len        = int(floor(round(self.__dit_len/period_len)*period_len))
        self.__da_len         = (180.0 * self.__sample_rate) / (50.0 * float(self.__speed))
        self.__da_len         = int(floor(round(self.__da_len/period_len)*period_len))
        self.__char_pause_len = int(round((180 * self.__sample_rate) / (50.0 * float(self.__eff_speed))))
        self.__word_pause_len = int(round((480* self.__sample_rate) / (50.0 * float(self.__eff_speed))))
        self.__buffer_len     = self.__dit_len*2

        # calculate dit-sample:
        self.__dit_sample = bytes()
        w = 2.0*pi * self.__frequency/self.__sample_rate
        for i in range(self.__dit_len):  # calc sine tone
            if i <= period_len:
                value = float(i)/period_len * self.__volume*sin(omega * float(i))
            elif i >= self.__dit_len-period_len:
                value = float(self.__dit_len-i)/period_len * self.__volume * sin(omega * float(i))
            else:
                value = self.__volume * sin(omega * float(i))
            self.__dit_sample += struct.pack("<h", int(value))
        self.__dit_sample += struct.pack("<h", 0)*self.__dit_len

        #calculate da-sample:
        self.__da_sample = bytes()
        for i in range( self.__da_len ):
            if i <= period_len:
                value = float(i)/period_len * self.__volume*sin(omega * float(i))
            elif i >= self.__da_len-period_len:
                value = float(self.__da_len-i)/period_len * self.__volume * sin(omega * float(i))
            else:
                value = self.__volume * sin(omega * float(i))
            self.__da_sample += struct.pack("<h", int(value))
        self.__da_sample += struct.pack("<h", 0)*self.__dit_len

        # "calculate" pause-sample:
        self.__cpause_sample = struct.pack("<h", 0)*self.__char_pause_len
        self.__wpause_sample = struct.pack("<h", 0)*self.__word_pause_len

        # set buffer size:
        self.__pcm.setperiodsize(self.__buffer_len)
        self.__sample_lock.release()


    def run(self):
        """ Internal used method! This method will be called in a new threead 
            by the constructor. """
        # main thread-loop
        while self.__running:        
            self.__pause.wait()
            if not self.__running: return
            
            # if there is no ChararcterSource -> pause; continue     
            if not (hasattr(self.__char_source, "__iter__") or isinstance(self.__char_source, basestring)):
                self.__pause.clear()
                continue
   
            # lock source and get char from source
            self.__char_source_lock.acquire()

            # set framecount = 0
            self.__frame_counter  = 0

            for char in self.__char_source:
                # if " " send a word-pause
                if char == " ":
                    self._send_word_pause()

                # if char is in table -> send each symbol
                elif char in self.__morse_table.keys():
                    for sym in self.__morse_table[char]:
                        if sym == ".": 
                            self._send_dit()
                        elif sym == "-": 
                            self._send_da()
                    self._send_char_pause()
               
                # if char is not known
                else: 
                    self.__logger.warning("Unknown char '%s'"%char);  
                
                # it is importent to exit before calling __char_notifier
                # otherwise there will be a deadlock-if the GUI waits for
                # this thread to exit and self.__cahr_notifier waits for
                # the gui to went into idle 
                if not self.__running:
                    break 
 
                if hasattr(self.__char_notifier, "__call__"):
                    self.__char_notifier(char)

                if not self.__pause.isSet():
                    break

            # done, unlock source
            self.__char_source_lock.release() 

            # finnaly go into pause
            if self.__pause.isSet():
                self.__pause.clear()  


    def set_char_source(self, src):
        """ Defines the char source. A char-source can be any iteratable over 
            chars. 

            Note; If this method gets called while the engine is running, this
            method will set the engine into pause-mode and restart it after 
            the char-source was seted. This may reset the internal 
            frame-counter."""
        # save old pause-state
        is_set = self.__pause.isSet()
        # set thread into pause:
        self.__pause.clear()

        # reset char source
        self.__char_source_lock.acquire()
        self.__char_source = src
        self.__char_source_lock.release()

        #reset state    
        if is_set: self.__pause.set()                          


    def stop(self):
        """ Stops the engine. This method signals the thread to exit and waits
            for join. """
        self.__running = False
        if not self.__pause.isSet():
            self.__pause.set()
        time.sleep(0.00001)
        self.join()


    def is_running(self):
        """ Returns True if the thread is sill alife. """
        return self.__running

    
    def is_paused(self):
        """ Returns True if the thread is paused. """
        return not self.__pause.isSet()


    def set_pause(self, paused=True):
        """ This method pauses the thread or wakes it up. """
        if paused:
            self.__pause.clear()
        else:
            self.__pause.set()
        time.sleep(0.00001)


    def set_notifier(self, notifier):
        """ This method can be used to specifiy a callable, that will get each
            char right after it was send. """
        self.__char_notifier = notifier


    def set_volume(self, volume):
        """ Resets the volume. """
        assert volume <= 0
        self.__volume = 10.0**(float(volume)/20.0) * 32767.0
        self.__generate_samples()

    def set_speed(self, speed):
        """ Resets the speed of coding. """
        self.__speed = speed
        self.__generate_samples()

    def set_eff_speed(self, speed):
        """ Resets the effective coding speed. """
        self.__eff_speed = speed
        self.__generate_samples()

    def set_frequency(self, freq):
        """ Specifies the tone-frequency. """
        self.__frequency = freq
        self.__generate_samples()

    
    def get_framecount(self):
        """ Returns the number of frames send since the last wakeup.
            Each time the thread wakesup from pause this counter will be set 
            to 0. """
        return self.__frame_counter

    def get_time(self):
        """ Returns the elapsed seconds since last wakeup from pause-mode. """
        return self.__frame_counter / self.__sample_rate


    def _send_data(self, data): 
        """ internal used method to play the precalculated samples. """
        # XXX Sorry this code looks ugly but alsaaudio does not work well with
        # GUI apps if the PCM is opened with PCM_NORMAL (blocking). So i need 
        # to handle playback in PCM_NONBLOCK mode. Unless data is written into
        # the buffer, the thread will sleep the half time it takes to playback
        # the written data. If the buffer was full (l == 0) we sleep the half 
        # time to playback the whole buffer. (buffersize is determed by 
        # dit-len)
        
        idx = 0
        # unless all data was written
        while idx < len(data):
            l = 0
            while l == 0:   # while no data send:
                # lock sample and write data
                self.__sample_lock.acquire()
                l = self.__pcm.write(data[idx:])
                self.__sample_lock.release()
            
                # on error
                if l < 0:
                    self.__logger.error("Error while write to pcm")
                    return
                
                # time to play written data
                t = float(l)/self.__sample_rate 
            
                if t > 0:       
                    # if data was written: sleep the half time to play these 
                    time.sleep(t/2)
                else:
                    # else sleep a half dit-length
                    time.sleep(float(self.__dit_len)/(2.0*self.__sample_rate))
            # update counters 
            idx += l*2
            self.__frame_counter += l


    def _send_dit(self):
        self._send_data(self.__dit_sample)

    def _send_da(self):
        self._send_data(self.__da_sample)
   
    def _send_char_pause(self):
        self._send_data(self.__cpause_sample)

    def _send_word_pause(self):
        self._send_data(self.__wpause_sample)



import wave

class WaveWriter:
    def __init__(self, filename):
        self.__wav = wave.open(filename, "wb")
        
    def __del__(self):   
        self.__wav.close()
 
    def setchannels(self, channels):
        self.__wav.setnchannels(channels)

    def setformat(self, format):
        self.__wav.setsampwidth(2)

    def setrate(self, rate):
        self.__wav.setframerate(rate)

    def setperiodsize(self, period): pass

    def write(self, data):
        self.__wav.writeframes(data)



""" alsamorse provides a simple to use morse-engine using alsaaudio for 
sound-output. 

This pachage contains the L{Morse} class. This class implements the engine. 
Additionally there is a L{KochLesson} class that can be used as a char-source 
for Morse. """

from .morse import Morse
from .koch import KochLesson 



from random import randint, choice


class KochLesson:
    chars = ['k','m','r','s','u','a','p','t','l','o','w',
             'i','.','n','j','e','=','f','0','y','v',',',
             'g','5','/','q','9','z','h','3','8','b','?',
             '4','2','7','c','1','d','6','x',
             'KA',"VE","VA"]

    def __init__(self, lesson, prefer_last=False, chars_ignored=""):
        self.set_lesson( lesson )
        self.__curr_group_size = randint(1,8)
        self.__char_of_grp     = 0
        self.__chars_send      = 0
        self.__prefer_last     = prefer_last
        self.__chars_ignored   = chars_ignored


    def __iter__(self):
        return self


    def __next__(self):
        c = self._get_next_char()
        while c in self.__chars_ignored and c != " ":
            c = self._get_next_char()
        self.__chars_send += 1
        self.__char_of_grp += 1
        return c


    def _get_next_char(self):
        # if group is complete: set next groupsize, 
        # reset counter and return a pause
        if self.__curr_group_size <= self.__char_of_grp:
            self.__curr_group_size = randint(1,8)
            self.__char_of_grp = -1
            return " "

        # increment counter and return a random char for current lesson
        if self.__prefer_last:
            def exp_prop(lst):
                if len(lst) <= 1: return lst
                return lst+exp_prop(lst[len(lst)//2:])
            return choice(exp_prop(self.chars[:self.__lesson]))
        else:
            return choice(self.chars[:self.__lesson])


    def get_chars_send(self):
        return self.__chars_send

    def reset_char_counter(self):
        self.__char_of_grp = 0
        self.__chars_send  = 0

    def get_num_lessons(self):
        return len(self.chars);

    def get_lesson(self):
        return self.__lesson
    
    def set_lesson(self, lesson):
        if not lesson in range(2,self.get_num_lessons()+1): 
            raise Exception("There are only {0} lessons".format(self.get_num_lessons()))
        self.__lesson = int(lesson)


    def get_preferlast(self):
        return self.__prefer_last

    def set_preferlast(self, prefer):
        self.__prefer_last = prefer

    def set_ignore_chars(self, chars):
        self.__chars_ignored = chars



#!/usr/bin/python

from distutils.core import setup

desc = """ Morse-tutor for Linux using the Koch-method. """

long_desc = """ This morse-tutor/trainer uses the Koch-method which is known to
be the fastest way to learn morse-code. 

KochMorse has a modern, easy to use Gtk3 user-interface. It also uses 
alsaaudio for sound-output."""

setup ( name = 'KochMorse',
        version = '2.0.0',
        description = desc,
        long_description = long_desc,
        author = 'Hannes Matuschek',
        author_email = 'hmatuschek@gmail.com',
        url = 'http://kochmorse.googlecode.com',

        classifiers = ['Development Status :: 5 - Production/Stable',
                       'Environment :: X11 Applications :: GTK',
                       'Intended Audience :: Education',
                       'Intended Audience :: End Users/Desktop',
                       'License :: OSI Approved :: GNU General Public License (GPL)',
                       'Operating System :: POSIX :: Linux',
                       'Programming Language :: Python :: 3',
                       'Topic :: Communications :: Ham Radio'], 

        packages = ['KochMorse','KochMorse.alsamorse'],
        requires = ['alsaaudio', 'gi'],
        scripts = ['kochmorse'],
        package_data = {'KochMorse' : ['*.ui']},
        
        data_files = [('/usr/share/applications',['kochmorse.desktop']),
                      ('/usr/share/KochMorse',   ['icon.svg']),
                      ('/usr/share/KochMorse/i18n', ['i18n/KochMorse.pot', 'i18n/de_DE.po']),
                      ('/usr/share/KochMorse/i18n/de/LC_MESSAGES', ['i18n/de/LC_MESSAGES/KochMorse.mo']) ] )                



import configparser
import os.path
import re


class Singleton(type):
    """ This class implement the singleton pattern for other classes. """
    def __init__(mcs, *args):
        type.__init__(mcs, *args)
        mcs._instance = None

    def __call__(mcs, *args):
        if mcs._instance is None:
            mcs._instance = type.__call__(mcs, *args)
        return mcs._instance



class Config:
    
      
    __metaclass__ = Singleton
    def __init__(self):
        self._options = {'lessontime': 5,
                         'countdown': 5,
                         'speed': 25,
                         'effspeed': 12,
                         'frequency': 750,
                         'volume': 0,
                         'lesson': 2,
                         'preferlast': False,
                         'ignorechars': ''}

        self._bool_opts  = ['preferlast']
        self._int_opts   = ['lessontime', 'countdown', 'speed', 'effspeed', 'frequency',
                            'lesson']
        self._float_opts = ['volume']
        self._str_opts   = ['ignorechars']
        self.__filename = os.path.expanduser("~/.kochmorse")

        if os.path.exists(self.__filename):
            self.load()
        else:
            self.save()


    def load(self):
        cfg = configparser.SafeConfigParser()
        cfg.read([self.__filename])
        
        if not cfg.has_section('basic'):
            return

        for option in self._options.keys():
            if not cfg.has_option('basic',option):
                continue
            if option in self._float_opts:
                self._options[option] = cfg.getfloat('basic', option)
            elif option in self._bool_opts:
                self._options[option] = cfg.getboolean('basic', option)
            elif option in self._int_opts:
                self._options[option] = cfg.getint('basic', option)
            else:
                self._options[option] = cfg.get('basic', option)
                    

    def save(self):
        cfg = configparser.SafeConfigParser()
        cfg.read([self.__filename])
        
        if not cfg.has_section('basic'):
            cfg.add_section('basic')
        
        for (key, value) in self._options.items():
            if key in self._float_opts:
                value = str(float(value))
            elif key in self._bool_opts:
                value = str(bool(value))
            elif key in self._int_opts:
                value = str(int(value))
            elif key in self._str_opts:
                value = str(value)
                    
            cfg.set('basic', key, value)

        cfg.write(open(self.__filename, "w"))
    

    def __getattr__(self, name):
        m = re.match("^opt_(.+)$", name)
        if not m: 
            return object.__getattribute__(self, name)
      
        return self._options[m.group(1)]


    def __setattr__(self, name, value):
        m = re.match("^opt_(.+)$", name)

        if not m:
            self.__dict__[name] = value
        else: 
            self._options[m.group(1)] = value



#!/usr/bin/python

from gi.repository import GObject, Gdk, Gtk
import time 
import os.path

from . import alsamorse
from .propdialog import PropertyDialog
from .conffile import Config





class KochMorse:

    def __init__(self):
        self.__gladefile = os.path.join(os.path.dirname(__file__), "kochmorse.ui")
        self.__xml = Gtk.Builder()
        self.__xml.add_from_file(self.__gladefile)
        self.__window     = self.__xml.get_object("window1")
        self.__textview   = self.__xml.get_object("textview")
        self.__statusbar  = self.__xml.get_object("statusbar")
        self.__speedspin  = self.__xml.get_object("speedspin")
        self.__effspeed   = self.__xml.get_object("effspeedspin")
        self.__lessonspin = self.__xml.get_object("lessonspin")
        self.__freqspin   = self.__xml.get_object("freqspin")
        self.__timelabel  = self.__xml.get_object("timelabel")
        self.__playbutton = self.__xml.get_object("playbutton")
        self.__volume     = self.__xml.get_object("volume")

        # load config and set entries
        self.__config = Config()
        self.__lesson_start = None
  
        self.__speedspin.set_value(self.__config.opt_speed)
        self.__effspeed.set_range(10, self.__config.opt_speed)
        self.__effspeed.set_value(self.__config.opt_effspeed)
        self.__lessonspin.set_value(self.__config.opt_lesson)
        self.__freqspin.set_value(self.__config.opt_frequency)
        self.__volume.set_value(self.__config.opt_volume)
 
        self.__engine = alsamorse.Morse(frequency = self.__config.opt_frequency, 
                                        speed = self.__config.opt_speed, 
                                        eff_speed = self.__config.opt_effspeed, 
                                        volume = self.__config.opt_volume)
        self.__lesson = alsamorse.KochLesson(self.__config.opt_lesson, self.__config.opt_ignorechars)
        self.__engine.set_char_source(self.__lesson)
        self.__engine.set_notifier(self.on_char_send)

        self.__lessonspin.set_range(2, self.__lesson.get_num_lessons())

        self.__xml.connect_signals({
                'on_playbutton_toggled': self.on_play_toggled,
                'on_quit': self.on_quit,
                'on_info_show': self.on_info_show,
                'on_propbutton_clicked': self.on_propbutton_clicked,
                'on_volume_changed': self.on_volume_changed,
                'on_speed_changed': self.on_speed_changed,
                'on_eff_speed_changed': self.on_eff_speed_changed,
                'on_lesson_changed': self.on_lesson_changed,
                'on_freq_changed': self.on_freq_changed})

        GObject.timeout_add(500, self.on_update_time)
        

    def on_play_toggled(self, button):
        if button.get_active():
            # clear char counter and textview
            self.__lesson.reset_char_counter()
            self.__textview.get_buffer().set_text("")
            self.__lesson_start = time.time()
        else:
            self.__engine.set_pause(True)
            self.__lesson_start = None


    def on_info_show(self, button):
        info_file = os.path.join(os.path.dirname(__file__), "infodialog.ui")
        info_xml = Gtk.Builder()
        info_xml.add_from_file(info_file);
        info_dialog = info_xml.get_object("infodialog")
        info_dialog.run()
        info_dialog.destroy()

    
    def on_propbutton_clicked(self, button):
        dlg = PropertyDialog()
        dlg.run()
        self.__lesson.set_preferlast(self.__config.opt_preferlast)
        self.__lesson.set_ignore_chars(self.__config.opt_ignorechars)


    def on_quit(self, window):
        self.__engine.stop()
        Gtk.main_quit()


    def on_char_send(self, char):
        Gdk.threads_enter()
        self.__textview.get_buffer().insert_at_cursor(char)        
        self.__statusbar.push(1, "%i chars send"%self.__lesson.get_chars_send())
        Gdk.threads_leave()

    
    def on_update_time(self):
        # if playbutton is pressed:
        if self.__lesson_start:
            # calc sec since start-button clicked
            tdiff = time.time()-self.__lesson_start-self.__config.opt_countdown
            
            if tdiff < 0:
                # while countdown:
                self.__timelabel.set_markup("<tt><big><b>%5i</b></big></tt>"%(tdiff))
            else:
                # after countdown
                if self.__engine.is_paused(): self.__engine.set_pause(False)
                # get and display sec since start:
                t = self.__engine.get_time()
                self.__timelabel.set_markup("<tt><big><b>%02i:%02i</b></big></tt>"%(int(t)/60,t%60))
            
                # check if lessontime is elapsed
                if t >= self.__config.opt_lessontime*60:
                    self.__playbutton.set_active(False)

        return True    


    def on_volume_changed(self, widget):
        self.__config.opt_volume = float(widget.get_value())
        self.__engine.set_volume( widget.get_value() )
        self.__config.save()


    def on_speed_changed(self, widget):
        self.__config.opt_speed = int(widget.get_value())
        self.__engine.set_speed(widget.get_value())
        self.__effspeed.set_range(10, self.__config.opt_speed)
        
        if self.__effspeed.get_value() > self.__config.opt_speed:
            self.__effspeed.set_value(self.__config.opt_effspeed)
            
        self.__config.save()
           
 
    def on_eff_speed_changed(self, widget):
        self.__config.opt_effspeed = int(widget.get_value())
        self.__engine.set_eff_speed( widget.get_value() )
        self.__config.save()


    def on_lesson_changed(self, widget):
        self.__config.opt_lesson = int(widget.get_value())
        self.__lesson.set_lesson(widget.get_value())
        self.__config.save()


    def on_freq_changed(self, widget):    
        self.__config.opt_frequency = float(widget.get_value())
        self.__engine.set_frequency(widget.get_value())
        self.__config.save()



